<?php

namespace Drupal\tellstick\Plugin\Block;

use Drupal\Component\Utility\SortArray;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\tellstick\TellstickSensorHelper;

/**
 * Provides a 'Tellstick' block.
 *
 * @Block(
 *   id = "telldus_block",
 *   admin_label = @Translation("Tellstick sensors"),
 * )
 */
class TellstickBlock extends BlockBase {

  /**
   * The Tellstick Live Service.
   *
   * @var \Drupal\tellstick\TelltickLiveService
   */
  protected $tellstickLiveService;

  /**
   * The Tellstick Service.
   *
   * @var \Drupal\tellstick\TelltickService
   */
  protected $tellstickService;

  /**
   * The Config Factory.
   *
   * @var \Drupal\tellstick\TelltickService
   */
  protected $config;

  /**
   * TellstickLiveService constructor.
   *
   * Can't inject the services as this is extending BlockBase.
   *
   * {@inheritdoc}
   */
  public function __construct(array $configuration,
                                    $plugin_id,
                                    $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->tellstickLiveService =
      \Drupal::service('tellstick.live');
    $this->tellstickService =
      \Drupal::service('tellstick');
    $this->config =
      \Drupal::service('config.factory');
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $block[] = [];

    $sxe = $this->tellstickLiveService->getTellstickLiveSensors();

    if ($sxe->getName() == 'sensors') {

      $sensor_items = [];

      foreach ($sxe->children() as $child) {

        $sensors = $child->attributes();
        $id = (string) $sensors['id'];
        $sensorStored = $this->tellstickService->getTellstickSensorById($id);
        if ($sensorStored <> FALSE) {
          // Better to check if sensor is of the right Class.
          // Get the stored Ignored and weight values.
          $active = !$sensorStored->getIgnored();
          $weight = $sensorStored->getWeight();
        }
        else {
          $active = FALSE;
        }

        if ($active) {
          // Show only the sensors info if enabled.
          //
          // TODO: Check if there is <data> for the sensor.
          // If not, then skip the sensor.
          $sensor = $this->tellstickLiveService->getTellstickLiveDataById($id);

          $items = [];
          $temperature = 0;
          $humidity = 0;
          foreach ($sensor->children() as $key) {
            if ($key->getName() == 'data') {

              /* Special treatment for the total counter */
              if ($key['name'] == 'rtot') {
                $since_midnight =
                  TellstickSensorHelper::variableSet(
                    (string) $sensors['id'],
                    'rain_midnight',
                    NULL);
                $items[] = [
                  '#markup' => TellstickSensorHelper::sensorPost('rday',
                    (double) $key['value'] - $since_midnight),
                  '#wrapper_attributes' => [
                    'class' => ['sensor_value sensor_value_rday'],
                  ],
                ];
              }

              /* Calculate the dew point */
              if ($key['name'] == 'temp') {
                $temperature = (double) $key['value'];
              }
              if ($key['name'] == 'humidity') {
                $humidity = (double) $key['value'];
              }
              if (($temperature <> 0) && ($humidity <> 0)) {
                $items[] = [
                  '#markup' => TellstickSensorHelper::sensorPost('dewpt',
                    round(TellstickSensorHelper::dewPoint(
                    $temperature,
                    $humidity), 1)),
                  '#wrapper_attributes' => [
                    'class' => ['sensor_value sensor_value_dewpt'],
                  ],
                ];
                $temperature = 0;
                $humidity = 0;
              }

              /* Handle the rest of the data */
              if (empty($key['scale'])) {
                $key['scale'] = NULL;
              }
              $items[] = [
                '#markup' => TellstickSensorHelper::sensorPost(
                  $key['name']->__toString(),
                  $key['value']->__toString(),
                  $key['scale']->__toString()
                ),
                '#wrapper_attributes' => [
                  'class' => [
                    'sensor_value sensor_value_' . (string) $key['name'],
                  ],
                ],
              ];
            }
          }

          // Add class tellstick_low_battery if (string)$sensors['battery']
          // is below threshold.
          $sensor_class = 'tellstick_sensor tellstick_sensor_' .
            (string) $sensors['id'];
          if ((string) $sensorStored->getBattery() <
            $this->config
              ->get('tellstick.sensor_settings')
              ->get('battery_level')) {

            $sensor_class .= ' tellstick_low_battery';
          }

          // Add class tellstick_old_data if (string)$sensors['lastUpdated']
          // is below threshold.
          if ($sensors['lastUpdated']->__toString() <
           date('U') - $this->config
             ->get('tellstick.sensor_settings')
             ->get('sensor_timeout')) {
            $sensor_class .= ' tellstick_old_value';
          }
          $sensor_items[] = [
            'children' => [
              '#theme' => 'item_list',
              '#title' => $sensors['name'] .
              ' (' . $sensors['clientName'] . ') ',
              '#items' => $items,
              '#list_type' => 'ul',
              '#wrapper_attributes' => ['id' => 'sensor'],
              '#attributes' => ['class' => $sensor_class],
            ],
            '#weight' => $weight,
          ];
        }
      }
      uasort($sensor_items, [SortArray::class, 'sortByWeightProperty']);
      $block['#theme'] = 'item_list__tellstick';
      $block['#wrapper_attributes'] = ['id' => 'tellsticksensors'];
      $block['#empty'] = 'No sensors found';
      $block['#list_type'] = 'ol';
      $block['#items'] = $sensor_items;
      $block['#attached'] = ['library' => 'tellstick/tellstick.block'];
      $block['#cache'] = ['max-age' => '300'];
    }
    return $block;
  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account) {
    return AccessResult::allowedIfHasPermission($account, 'view tellstick');
  }

}
