<?php

namespace Drupal\tellstick;

require_once 'HTTP/OAuth/Consumer.php';

define('REQUEST_URI', 'http://api.telldus.com/xml');
define('REQUEST_URI_S', 'https://api.telldus.com/xml');

use HTTP_OAuth_Consumer;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * Class Telsstick live service.
 *
 *  Handles live requests for Tellstick Sensors.
 */
class TellstickLiveService {

  /**
   * The type Config Factory injected into the service.
   *
   * @var Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The type Config Factory injected into the service.
   *
   * @var Drupal\Core\Messenger\MessengerInferface
   */
  protected $messenger;

  /**
   * Constructor.
   *
   * @param Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param Drupal\Core\iMessenger\MessengerInterface $messenger
   *   The Messenger Interface.
   */
  public function __construct(ConfigFactoryInterface $config_factory,
                              MessengerInterface $messenger) {
    $this->configFactory = $config_factory;
    $this->messenger = $messenger;
  }

  /**
   * Get Tellstick Credentials.
   *
   * @return Drupal\Core\Config\ImmutableConfig
   *   An array of Tellstick Authentication Credentials.
   */
  public function getTellstickCredentials() {
    return $this->configFactory->get('tellstick.settings');
  }

  /**
   * Get all Tellstick Live Sensors Currently active.
   *
   * @return Drupal\tellstick\Entity\TellstickData[]
   *   An array of Tellstick Data Items.
   */
  public function getTellstickLiveSensors() {
    $consumer = $this->connect();
    $includeSensors = $this->configFactory
      ->get('tellstick.settings')
      ->get('include_hidden');
    $params = ['includeIgnored' => $includeSensors];
    $response = $consumer->sendRequest($this->getAccessMethod() .
      '/sensors/list',
      $params,
      'GET'
    );
    $sxe = new \SimpleXMLElement($response->getBody());
    return $sxe;
  }

  /**
   * Get all Tellstick Live Data.
   *
   * @return Drupal\tellstick\Entity\TellstickData[]
   *   An array of Tellstick Data Items.
   */
  public function getTellstickLiveData() {
    $consumer = $this->connect();
    $includeSensors = $this->configFactory
      ->get('tellstick.settings')
      ->get('include_hidden');
    $params = [
      'includeIgnored' => $includeSensors,
      'includeValues' => 1,
    ];
    $response = $consumer->sendRequest($this->getAccessMethod() .
      '/sensors/list',
      $params,
      'GET'
    );
    $sxe = new \SimpleXMLElement($response->getBody());
    return $sxe;
  }

  /**
   * Get info on the sensor with sensorId.
   */
  public function getTellstickLiveSensorInfo($sensorId) {
    $sensors = $this->getTellstickLiveSensors();
    foreach ($sensors->children() as $child) {
      $sensor = $child->attributes();
      if ($sensor['id'] == $sensorId) {
        return $sensor;
      }
    }
    return FALSE;
  }

  /**
   * Get Tellstick Live Sensors Data by $id.
   *
   * @return Drupal\tellstick\Entity\TellstickData[]
   *   An array of Tellstick Data Items.
   */
  public function getTellstickLiveDataById($id) {
    $consumer = $this->connect();
    $params = ['id' => $id];
    $response = $consumer->sendRequest($this->getAccessMethod() .
      '/sensor/info',
      $params,
      'GET'
    );
    return new \SimpleXMLElement($response->getBody());
  }

  /**
   * Request a Consumer object.
   *
   * @return \HTTP_OAuth_Consumer
   *   A HTTP_OAuth_Consumer
   */
  private function connect() {
    $config = $this->getTellstickCredentials();

    if (!$config->get('public_key') ||
      !$config->get('private_key')) {
      $url = Url::fromRoute('tellstick.setting_form');
      $link = Link::fromTextAndUrl('not set', $url);
      $message = t('The Tellstick Credentials are @link!',
        ['@link' => $link->toString()]);
      $this->messenger->addError($message);
    }
    return new HTTP_OAuth_Consumer(
      $config->get('public_key'),
      $config->get('private_key'),
      $config->get('token'),
      $config->get('token_secret')
    );
  }

  /**
   * Custom content function.
   *
   * Return a path for accessing the Telldus Live! server.
   *
   * @return RequestUri
   *   A complete URI
   */
  private function getAccessMethod() {
    if ($this->configFactory
      ->get('tellstick.settings')->get('access_method') == 0) {
      return constant('REQUEST_URI');
    }
    else {
      return constant('REQUEST_URI_S');
    }
  }

}
