<?php

namespace Drupal\tellstick;

use Drupal\Component\Utility\Html;

/**
 * Helper class for Telsstick Sensor data handling.
 */
class TellstickSensorHelper {

  /**
   * The type Config Factory injected into the service.
   *
   * @var Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructor.
   *
   * @param Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
  }

  /**
   * Format a sensor data post with correct units etc.
   *
   * @return string
   *   A formated sensor post.
   */
  public static function sensorPost($raw_name = '', $raw_value = 0, $raw_scale = NULL) {
    global $conf;
    $config = \Drupal::service('config.factory');

    $name = Html::escape($raw_name);
    $value = Html::escape($raw_value);
    $scale = Html::escape($raw_scale);

    $content = '';

    switch ($name) {
      case 'rrate':
        $content .= t('Rain: @value mm/h', ['@value' => $value]);
        break;

      case 'rday':
        $content .= t('Since midnight: @value mm', [
          '@value' => number_format((double) $value, 1),
        ]
        );
        break;

      case 'rtot':
        $content .= t('Total: @value mm', ['@value' => $value]);
        break;

      case 'temp':
        $value = ($config->get('temperature_unit') == '2') ?
          $value + 273.15 : $value;
        $content .= t('Temperature: @value &deg;@unit', [
          '@value' => $value,
          '@unit' => TellstickSensorHelper::getTemperatureUnit(),
        ]);
        break;

      case 'humidity':
        $content .= t('Humidity: @value %', ['@value' => $value]);
        break;

      case 'dewpt':
        $value = ($config->get('temperature_unit') == '2') ?
          $value + 273.15 : $value;
        $content .= t('Dew point: @value &deg;@unit', [
          '@value' => $value,
          '@unit' => TellstickSensorHelper::getTemperatureUnit(),
        ]);
        break;

      case 'wavg':
        $content .= t('Wind avg. @value m/s', ['@value' => $value]);
        break;

      case 'wgust':
        $content .= t('Wind gust. @value m/s', ['@value' => $value]);
        break;

      case 'wdir':
        $value = ((double) $value + (double) $config
          ->get('tellstick.sensor_settings')
          ->get('offset')) % 360;
        $cardinal = TellstickSensorHelper::getCardinal($value);
        $content .= t('Wind dir. @value', ['@value' => $cardinal]);
        break;

      case 'battery':
        // This is an integer converted to double...
        if ($value < 253.9) {
          $content .= t('Battery: @value%',
            ['@value' => (int) ($value * 100 / 253)]);
        }
        /* else {
        $content .= t('Battery') . ': ' . '&nbsp;Unknown';
        } */
        break;

      case 'watt':
        switch ($scale) {
          case '':
            $content .= t('Acc. Energy: @value kWh', ['@value' => $value]);
            break;

          case '1':
            $content .= t('Curr. Energy: @value kVAh', ['@value' => $value]);
            break;

          case '2':
            $content .= t('Curr. Power: @value W', ['@value' => $value]);
            break;

          case '3':
            $content .= t('Puls: @value', ['@value' => $value]);
            break;

          case '4':
            $content .= t('Voltage: @value V', ['@value' => $value]);
            break;

          case '5':
            $content .= t('Current: @value A', ['@value' => $value]);
            break;

          case '6':
            $content .= t('Power factor: @value', ['@value' => $value]);
            break;

        }
        break;

      default:
        $content .= t('@name: @value', [
          '@name' => $name,
          '@value' => $value,
        ]);

    }
    return $content;
  }

  /**
   * Return the cardinal for each direction.
   *
   * @return string
   *   The Cardinal as a string
   */
  private static function getCardinal($direction) {
    $normalized = (integer) round(($direction + 11.25) / 22.5,
        1, PHP_ROUND_HALF_UP);

    switch ($normalized) {
      case 0:
        $normalized = t('N');
        break;

      case 1:
        $normalized = t('NNE');
        break;

      case 2:
        $normalized = t('NE');
        break;

      case 3:
        $normalized = t('ENE');
        break;

      case 4:
        $normalized = t('E');
        break;

      case 5:
        $normalized = t('ESE');
        break;

      case 6:
        $normalized = t('SE');
        break;

      case 7:
        $normalized = t('SSE');
        break;

      case 8:
        $normalized = t('S');
        break;

      case 9:
        $normalized = t('SSW');
        break;

      case 10:
        $normalized = t('SW');
        break;

      case 11:
        $normalized = t('WSW');
        break;

      case 12:
        $normalized = t('W');
        break;

      case 13:
        $normalized = t('WNW');
        break;

      case 14:
        $normalized = t('NW');
        break;

      case 15:
        $normalized = t('NNW');
        break;

      default:
        $normalized = t('Should not occur...');
    }
    return $normalized;
  }

  /**
   * Return the dew point for a given temperature and humidity.
   *
   * See http://en.wikipedia.org/wiki/Dew_point
   * Temperature i degrees C.
   *
   * @return dewPoint
   *   The dew point, FALSE if outside of formula limits
   */
  public static function dewPoint($temperature, $humidity) {

    $config = \Drupal::service('config.factory');
    $temperature =
      ($config->get('tellstick_temperatur_unit') == '1') ?
      /* Convert Fahrenheit to Celsius */
      ($temperature - 32) * 5 / 9 :
      $temperature;
    if (($temperature <= -30) || ($temperature >= 35)) {
      return FALSE;
    }
    if (($humidity <= 1) || ($humidity > 100)) {
      return FALSE;
    }
    $b = 17.67;
    $c = 243.5;
    $temperature = $c * TellstickSensorHelper::dewPointHelper($temperature, $humidity) /
      ($b - TellstickSensorHelper::dewPointHelper($temperature, $humidity));
    $temperature =
      ($config->get('tellstick_temperatur_unit') == '1') ?
      /* Convert Celcius to Fahrenheit */
      ($temperature * 9 / 5) + 32 :
      $temperature;
    return $temperature;
  }

  /**
   * Return the dew point for a given temperature and humidity.
   *
   * See http://en.wikipedia.org/wiki/Dew_point
   * Temperature i degrees C.
   *
   * @return dewPoint
   *   The dew point
   */
  private static function dewPointHelper($temperature, $humidity) {
    $b = 17.67;
    $c = 243.5;
    return log($humidity / 100) + ($b * $temperature) / ($c + $temperature);
  }

  /**
   * Return the mean temperature over day.
   *
   * NOT YET IMPLEMENTED!
   *
   * @return mean
   *   The temperature mean
   */
  private function calculateMeanTemp($day) {
    $mean = 0;
    return $mean;
  }

  /**
   * Return the current temperature unit.
   *
   * @return TemperatureUnit
   *   The temperature unit
   */
  private static function getTemperatureUnit() {

    $config = \Drupal::service('config.factory')
      ->get('tellstick.sensor_settings');
    $units = ['0' => 'C', '1' => 'F', '2' => 'K'];
    return $units[$config->get('temperature_unit')];
  }

  /**
   * Implements shared array using Drupals variables.
   *
   * @param string $id
   *   The id of the sensor.
   * @param string $name
   *   The name of the array index to set.
   * @param string $value
   *   The value to set name to.
   *
   * @return sensor
   *   The value
   */
  public static function variableSet($id, $name, $value = NULL) {
    $sensorService = \Drupal::service('tellstick');
    $sensor = $sensorService->getTellstickSensorById($id);
    if (!empty($sensor)) {
      if (isset($value)) {
        $sensor->setData($name, $value);
        $sensor->save();
      }
      return $sensor->getData($name);
    }
    else {
      return NULL;
    }
  }

}
